# weblog
from config import *

#####################

import xmlrpclib, os.path, os, stat, time, socket
from wxPython.wx import *

# functions

def config_fn(leaf):
	"returns the full path to a config file and creates the ~/.topicexchange dir if not found.  e.g. 'foo.txt' -> '/home/phil/.topicexchange/foo.txt'"
	te_dir = os.path.expanduser('~/.topicexchange')
	if not os.path.isdir(te_dir):
		os.mkdir(te_dir)
	return os.path.join(te_dir, leaf)

def read_cache():
	return eval(open(inf.posts_cache_fn).read())

def write_cache(cache):
	open(inf.posts_cache_fn, 'wt').write(`cache`)

# classes

class ite:
	"topic exchange rpc wrapper"
	
	def __init__(self):
		self.ite_rpc = xmlrpclib.Server(ite_rpc_url)
		self.channels = None
		
	def getchannels(self):
		if self.channels is None:
			channelfile = config_fn("channels.txt")
			if os.path.exists(channelfile) and (time.time() - os.stat(channelfile)[stat.ST_MTIME]) < 3600:
				# read from disk; it's cached and fairly recent
				self.channels = eval(open(channelfile).read())
			else:
				# get from server and save
				self.channels = self.ite_rpc.topicExchange.getChannels()['channels']
				open(channelfile, 'wt').write(`self.channels`)
		return self.channels

def firstbefore(s, bits, end):
	best = -1
	bestbit = None
	for bit in bits:
		v = s.rfind(bit, 0, end)
		if v != -1 and v > best:
			best = v
			bestbit = bit
	return (best, bestbit)

def firstafter(s, bits, start):
	best = -1
	bestbit = None
	for bit in bits:
		v = s.find(bit, start)
		print "find: %s, %d -> %d" % (`bit`, start, v)
		if v != -1 and (best == -1 or v < best):
			best = v
			bestbit = bit
	print "firstafter(s, %s, %d): returned %s" % (bits, start, `(best,bestbit)`)
	print "s[%d:] = %s" % (start, `s[start:]`)
	return (best, bestbit)

def splitdesc(desc):
	pos = desc.find(this_post_appears_msg)
	if pos != -1:
		# we already have a message there - have to get rid of it
		linestart, startvalue = firstbefore(desc.lower(), ('<p', '\n'), pos)
		if linestart == -1:
			# odd - the message is at the _start_ of the post
			linestart = 0
		lineend, endvalue = firstafter(desc.lower(), ('</p>', '\n'), pos)
		if lineend == -1:
			# at the end of the post, but trailing crs have been stripped
			lineend = len(desc)
		else:
			lineend += len(endvalue)
		descstart = desc[:linestart].rstrip()
		descend = desc[lineend:].lstrip()
	else:
		descstart = desc
		descend = ''
	return (descstart, descend)

def merge_topics_line(desc, topics_line):
	descstart, descend = splitdesc(desc)
	print "\told desc: %s" % `desc`
	print "\tstart: %s" % `descstart`
	print "\tend: %s" % `descend`
	newdesc = descstart + topics_line + descend
	print "\tnew desc: %s" % `newdesc`
	return newdesc

def test_merge_topics_line():
	topics_line1 = '\n\n<p align="right"><i>%s %s</i></p>' % (this_post_appears_msg,
			", ".join([
			'<a href="%st/%s/">%s</a>' % (ite_server, topic, topic) for topic in ('one', 'two', 'three')
			]))
	topics_line2 = '\n\n<p align="right"><i>%s %s</i></p>' % (this_post_appears_msg,
			", ".join([
			'<a href="%st/%s/">%s</a>' % (ite_server, topic, topic) for topic in ('four', 'five', 'six')
			]))
	
	print "testing merge_topics_line():"
	mydesc = 'line 1'
	print "mydesc:",`mydesc`
	d2 = merge_topics_line(mydesc, topics_line1)
	print "d2:",`d2`
	d3 = merge_topics_line(mydesc, topics_line2)
	print "d3:",`d3`
	d4 = merge_topics_line(d2, topics_line2)
	print "d4 (should == d3):",`d4`
	assert d3 == d4
	line2 = 'this is a test (second line)'
	d2 += line2
	d5 = merge_topics_line(d2, topics_line2)
	print "d5 (should == d3 + line2):",`d5`
	assert d3 + line2 == d5

	mydesc = '<P>good - the topic list at the end of that last post updated properly.</P>\n<P>now for a tricky test: a topic list in the <EM>middle</EM> of a post.&nbsp; steps to do this:</P>\n<OL>\n<LI>write a post (in "Radio") \n<LI>assign it some topics in the client \n<LI>edit the post and add some more text below the topic line \n<LI>assign it some more topics</LI></OL>\n<P>hopefully it should end up with the new topics showing in the topic line, which is now in the middle of the post.</P>\n<P align=right><I>This post appears in channels: <A href="http://home.topicexchange.com/t/montreal_quebec_canada/">montreal_quebec_canada</A>, <A href="http://home.topicexchange.com/t/new_topics_on_topicexchange/">new_topics_on_topicexchange</A>, <A href="http://home.topicexchange.com/t/new_webloggers/">new_webloggers</A></I></P>\n<P align=left>ok, this is #3 ...</P>'
	
	d6 = merge_topics_line(mydesc, topics_line2)
	assert d6.endswith('...</P>')

class PostFrame(wxFrame):
	"window containing info about a single post"
	
	def __init__(self, pid, parent):
		self.parent = parent
		wxFrame.__init__(self, None, -1, "topics for post %s" % pid)
		panel = wxPanel(self, -1)

		self.pid = pid
		self.unchecking = 0

		vbox = wxBoxSizer(wxVERTICAL)
		panel.SetSizer(vbox)
		panel.SetAutoLayout(true)
		
		self.post = inf.posts[pid]
		
		vbox.Add(wxStaticText(panel, -1, "Post content:"), 0, wxEXPAND)
		vbox.Add(wxTextCtrl(panel, -1, self.post['description'], style=wxTE_READONLY | wxTE_MULTILINE), 1, wxEXPAND)
		import stripper
		vbox.Add(wxStaticText(panel, -1, "Excerpt for the Topic Exchange:"), 0, wxEXPAND)
		self.excerpt = "".join(splitdesc(self.post['description']))
		#stripper.strip("".join(splitdesc(self.post['description']))).replace('\n', '<br/>\n')
		vbox.Add(wxTextCtrl(panel, -1, self.excerpt, style=wxTE_READONLY | wxTE_MULTILINE), 1, wxEXPAND)
		
		vbox.Add(wxStaticText(panel, -1, "Topic channels:"), 0, wxEXPAND)
		channels = inf.ts.getchannels()

		self.channel_selections = {}
		
		cache = self.GetCache()

		self.chanList = wxCheckListBox(panel, -1)
		chans = channels.items()
		chans.sort()
		itemid = 0
		for cname,cdata in chans:
			if not cname: continue
			#self.channel_selections[pid] = 
			self.chanList.Append(cname)
			if cname in cache['channels']:
				self.chanList.Check(itemid, 1)
			itemid += 1

		EVT_CHECKLISTBOX(self, self.chanList.GetId(), self.CheckClicked)
		vbox.Add(self.chanList, 5, wxEXPAND)
		
		self.topicsText = wxStaticText(panel, -1, "")
		vbox.Add(self.topicsText, 0, wxEXPAND)
		self.ShowTopics()
		
		buttons = wxBoxSizer(wxHORIZONTAL)
		def btn(name, cmd=None):
			b = wxButton(panel, -1, name)
			buttons.Add(b)
			if cmd is not None:
				EVT_BUTTON(self, b.GetId(), cmd)
			return b.GetId()
		btn("Clear all topics", cmd=self.ClearAll)
		btn("Apply", cmd=self.ApplyClick)
		#btn("Ping all topics", cmd=self.PingAll)
		#btn("Add topics line", cmd=self.AddTopicsLine)
		btn("OK", cmd=self.OkClick)
		btn("Cancel", cmd=self.CancelClick)
		vbox.Add(buttons, 0, wxALIGN_RIGHT)

		panel.GetSizer().Fit(panel)
		panel.GetSizer().SetSizeHints(panel)
		
		self.SetSize((500,300))
		self.Centre()
		self.Maximize()

	def AddTopicsLine(self):
		topics_line = '\n\n<p align="right"><i>%s %s</i></p>' % (this_post_appears_msg,
			", ".join([
			'<a href="%st/%s/">%s</a>' % (ite_server, topic, topic) for topic in self.GetTopics()
			]))
		newdesc = merge_topics_line(self.post['description'], topics_line)
		inf.cached[self.pid]['newdesc'] = newdesc
		self.post['description'] = newdesc
		print "updating ..."
		inf.blog_rpc.metaWeblog.editPost(self.post['postid'], blog_login, blog_password, self.post, xmlrpclib.True)
		print "done."

	def GetCache(self):
		return inf.cached[self.pid]
	
	def PingAll(self):
		topics = self.GetTopics()
		c = self.GetCache()
		if not c.has_key('pings'):
			c['pings'] = {}
		for topic in topics:
			print "pinging %s ..." % topic
			ret = inf.ts.ite_rpc.topicExchange.ping(topic, {
				'blog_name': inf.blogname,
				'title': get_title(self.post),
				'url': self.post['permaLink'],
				'excerpt': self.excerpt,
				'autoMunge': xmlrpclib.False,
			})
			del ret['flError']
			c['pings'][topic] = ret
			print ret
			write_cache(inf.cached)

	def ClearAll(self, evt):
		self.unchecking = 1
		for i in range(self.chanList.Number()):
			self.chanList.Check(i, 0)
		self.ShowTopics()
		self.unchecking = 0

	def ApplyClick(self, evt):
		self.PingAll()
		self.AddTopicsLine()
		self.GetCache()['channels'] = self.GetTopics()
		self.parent.SetTopicsText(self.pid)

	def OkClick(self, evt):
		self.ApplyClick(evt)
		self.Destroy()

	def CancelClick(self, evt):
		self.Destroy()

	def GetTopics(self):
		topics = []
		for i in range(self.chanList.Number()):
			if self.chanList.IsChecked(i):
				topics.append(self.chanList.GetString(i))
		return topics
		
	def ShowTopics(self):
		self.topicsText.SetLabel("Topics: " + ", ".join(self.GetTopics()))

	def CheckClicked(self, evt):
		if self.unchecking: return
		#print "checkbox clicked",evt,dir(evt)
		#print "checked?",evt.IsChecked()
		self.ShowTopics()

def get_title(r):
	return r.get('title', 'untitled')

class MainFrame(wxFrame):
	"main window - shows all posts"
	
	def __init__(self):
		wxFrame.__init__(self, None, -1, "annotate your posts", style=wxDEFAULT_FRAME_STYLE | wxRESIZE_BORDER)
		panel = wxPanel(self, -1)

		postno = 0
		
		vbox = wxBoxSizer(wxVERTICAL)
		panel.SetSizer(vbox)
		panel.SetAutoLayout(true)

		btnid = 10
	
		for r in inf.recent:
			pid = r['postid']
			title = get_title(r)
			
			hbox = wxBoxSizer(wxHORIZONTAL)
			hbox.Add(wxStaticText(panel, -1, "[%s] %s" % (pid, title)), 1, wxEXPAND)

			btnid += 1
			btn = wxButton(panel, btnid, "topics")
			r['button'] = btn
			self.SetTopicsText(pid)
			hbox.Add(btn, 0, wxEXPAND)
			EVT_BUTTON(self, btnid, (lambda x,pid=pid: self.topic_button_click(x,pid)))
			vbox.Add(hbox, 0, wxEXPAND)

		self.future_channels = {}
		
		panel.GetSizer().Fit(panel)
		panel.GetSizer().SetSizeHints(panel)
		
		self.SetSize((500,300))
		self.Centre()

	def SetTopicsText(self, postid):
		topics = inf.cached[postid]['channels']
       		topics_txt = 'topics'
       		if len(topics):
	       		topics_txt += ' (%d)' % len(topics)
		for r in inf.recent:
			if r['postid'] == postid:
				r['button'].SetLabel(topics_txt)

	def topic_button_click(self, evt, pid):
		#print "topic button",evt,pid
		p = PostFrame(pid, self)
		p.Show()		

class App(wxApp):
	def OnInit(self):
		top = MainFrame()
        	self.SetTopWindow(top)
        	top.Show()
        	return true

# globals

class inf:
	blog_rpc = xmlrpclib.Server(blog_server)
	posts_cache_fn = config_fn('posts.txt')
	cached = {}
	ts = ite()
	use_blogger = 0 # 0 = use metaweblog
	appkey = 'foo'

# main

def main():
	try:
		for b in inf.blog_rpc.blogger.getUsersBlogs(inf.appkey, blog_login, blog_password):
			print b
			inf.blogname = b['blogname']
			inf.blogid = b['blogid']
			break
	except socket.error:
		print "Socket error -- is your blogging software running?"
		raise
	print "blogid:",inf.blogid

	if os.path.exists(inf.posts_cache_fn):
		inf.cached = read_cache()

	inf.recent = inf.blog_rpc.metaWeblog.getRecentPosts(inf.blogid, blog_login, blog_password, 20)
	if 0: # that failed
		inf.use_blogger = 1
		inf.recent = inf.blog_rpc.blogger.getRecentPosts(inf.appkey, inf.blogid, blog_login, blog_password, 20)
	inf.posts = {}
	for r in inf.recent:
		print "-- post --"
		print r
		postid = r['postid']
		if inf.cached.has_key(postid):
			print "in cache!"
		else:
			inf.cached[postid] = {'channels': [], 'excerpt': ''}
		data = inf.cached[postid]
		inf.posts[postid] = r
		print

	print inf.posts.keys()

	App().MainLoop()

	write_cache(inf.cached)

if __name__ == '__main__':
	test_merge_topics_line()
	main()

